/*********************************************************************

 spie.v module

**********************************************************************

	  		     CONFIDENTIAL

 Copyright (C) 2003, 2004, Regents of the University of California,

 The information contained herein is the exclusive property
 of the VCL group and shall not be distributed, reproduced,
 or disclosed in whole or in part without prior written 
 permission of VCL.

 This work has been developed by members of the VLSI Computation Lab
 (VCL) in the Department of Electrical and Computer Engineering at
 the University of California at Davis. Contact: bbaas@ece.ucdavis.edu

**********************************************************************

 created on:	04/06/2005 
 created by:	Jeremy W. Webb
 last edit on:	07/17/2008 
 last edit by:	Jeremy W. Webb
 revision:	008
 comments:	Created.

		Added the structure, and coded up the next-state logic.
		Next change is to fix the sensitivity list in the 
		next-state logic always block, and add all of the 
		state machine guts.

                Added description of state machine states, and 
                finished the state machine design.        

                Finished the SPI portion. I still need to add the 
                config_clk and config_valid states to the FSM.

                Added instantiation template at the end.

                Fixed a bug with the split_done control signal. It wasn't
                getting reset to zero, so you had to do an SPI write twice
                or set the spi_start twice. Also fixed a few bugs in 
                the code.

                Modified for AsAP version 2.

                Modified for use with MicroBlaze.

**********************************************************************
 Asynchronous Array of simple Processors (AsAP)

 This module implements the serial peripheral interface (SPI)
 engine (master) for use in configuring the AsAP chip. 

 The interface consists of 7 signals:

 MOSI: Master Out, Slave In data output. This is data from the 
       SPI Master.

 MISO: Master In, Slave Out data input. This is data sent by
       the AsAP to the SPI Master.

 SCK: Serial Clock. Only active during serial transactions.

 CSn: Active Low Chip Select. This line idles in a high state.
      The Serial Address/Data is valid on MOSI when CSn is low.

 LOAD_EN: Active High Load Enable. This line idles in a 
	  low state. Data sent on MOSI is latched on a 
	  rising edge of LOAD_EN.

 The outgoing serial data is sent in 18-bit packets/symbols,
 and the parallel data is formated as follows:

  par[17:0]	par[18]		par[19]	  Description
 ------------------------------------------------------------
 Upper Addr     0b0             0b0        Upper Address Word
					   config[52:48]
 Lower Addr	0b0		0b1	   Lower Address Word
					   config[47:32]
 Upper Data	0b1		0b0	   Upper Data Word
					   config[31:16]
 Lower Data	0b1		0b1	   Lower Data Word
					   config[15:0]

 par[18] is used for determining if address or data is being
 sent. If par[18] = 0, then address is being sent. If 
 par[18] = 1, then data is being sent.

 par[19] is used for determining which part of the word is
 being sent: upper or lower. If par[19] = 0, then the upper
 part of the word is being sent. If par[19] = 1, then the 
 lower part of the word is being sent.

 Note the address bus of the processor configuration blocks
 only require 21-bits, so the upper 11-bits of the 16-bit
 address word are set to all zeros. The configuration blocks 
 know how to interpret the data based on the status of the 
 2 MSB bits of each 18-bit word. Each of the 4 serial 
 transactions outlined above are sent with a configuration 
 clock and a configuration valid signal to clock the parallel
 data into the AsAP processor configuration blocks.

 Address and Data is sent via an RS-232 interface from a 
 host PC. The address and data is formatted in 32-bit widths.
 The 32-bit words are split up into two 16-bit words and 
 sent in two 18-bit transactions. Data and Address are sent 
 in pairs of 32-bit words. Below is an example of the RS-232
 terminal interface:

        <com1> Addr: 00000A52
        <com1> Data: 0F0E0C09
        <com1> Addr: 0000FC54
        <com1> Data: 0C0A0E06        

 // --- State Machine Parameters ---
 The idle state initializes all state machine registers 
 and counters.

 The wait_data state waits until the start of configuration 
 data being sent from the RS-232 interface modules. Once the 
 start_spi flag has been set high, the state machine will 
 advance to the next state. The hold flag is set low 
 indicating to the RS-232 interface module that the SPI 
 Engine is ready to receive data. The Configuration Address 
 should be made available on the next clock edge, where it 
 will be stored in uppera/lowera.

 The split_data state stores the configuration address in registers
 uppera and lowera, and the configuration data in registers upperd 
 and lowerd. When it finishes storing the configuration data, it 
 sets the hold flag signaling to the RS-232 interface module that 
 it should stop sending configuration address and data pairs until 
 the current 4 SPI transactions are completed. The SPI transactions 
 consist of 20-bit transfers, thus we require 4 SPI transactions to 
 transfer 72-bits across the SPI interface. The upper and lower 
 data/address information are encoded into 20-bit chunks/symbols 
 that allow the AsAP chip to decode the 20-bit symbols for routing 
 the symbols to the appropriate locations.

 The load_shft_data state loads either the upper or lower address/data 
 into the SPI shift register. It first loads the upper address into the 
 spi_ld_data register, and advances to the next state. At the end of 
 the upper address SPI transaction, the lower address is loaded into
 the spi_ld_data register. At the end of the lower address SPI 
 transaction, the upper data is loaded into the spi_ld_data register.
 At the end of the upper data SPI transaction, the lower data is loaded
 into the spi_ld_data register. At the end of the lower data SPI 
 transaction, the state machine advances to the wait_data state where
 it waits for new data to restart the 4 SPI transactions.

 The enable_sck state enables the SPI clock (SCK). It provides two 
 SCK clock pulses prior to the SPI chip select (CSn) toggling low, and
 then advances to the next state. The two SCK clock pulses are 
 inherently provided just by enabling the SCK in this state and 
 immediately advancing to the enable_cs state. The CSn doesn't 
 toggle low until the next clock after it leaves the enable_cs state.

 The enable_cs state toggles the CSn low and advances to the next state.

 The wait_for_wr state waits for 20 SCK cycles to complete before 
 advancing to the next state to disable CSn. The total number of 
 system clocks elapsing during this wait state will be a power-of-two
 multiple of 20. For example, if SCK is equal to the system clock 
 divided 32, then we'll have to wait for 640 system clocks until 
 disabling CSn. It may be better and more hardware efficient to have 
 an external shift counter counting the number of shifts, and setting 
 a shift_done flag. Refer to adc_read.v for sample code.

 The disable_cs state toggles the CSn high, and advances to the next state.

 The set_load_en state toggles the spi_load_en signal high, then 
 advances to the load_shft_data.

 
**********************************************************************/

`timescale 10ps/1ps


module  spie   (// *** Inputs ***
                input   wire            clk_50,         // System Clock. (100 MHz)
                input   wire            rst_n,          // System Reset (Active Low)
                input   wire		MISO,	        // Master In Slave Out.
                input   wire    [39:0]  config_data,    // Configuration Data.       // DEAN WAS HERE
                input   wire    [39:0]  config_addr,    // Configuration Address.    // DEAN WAS HERE
                input   wire            start_spi,      // Start SPI Data Transfer.

		// *** Outputs ***
                // RS-232 Interface Ctrl
                output  reg             hold,           // Hold Off sourcing data. (Active High)
                output  reg             send,           // Ready for more data. (Active High)
                output  wire            miso_rdy,       // MISO Read Data is valid. (Active High)
                // Config Interface
                output  wire            config_clk,     // Configuration Clock.
                output  wire            config_valid,   // Configuration Valid.
                // SPI Interface
		output  wire		SCK,	        // SPI Clock (max: 20MHz).
		output  wire		CSn,   	        // Chip Select(Active Low).
		output  wire		MOSI,	        // Master Out Slave In.
		output  wire		LOAD_EN,        // Load Enable(Active Hi).
                output  reg     [19:0]  miso_read       // Loopback of MOSI data.     // DEAN WAS HERE
	       );

// *** Local Variable Declarations ***
// Local Parameter Declarations:
// --- State Machine Parameters ---
localparam      idle            =  14'h1;    // Initialize all control signals.
localparam      wait_data	=  14'h2;    // Wait for start of configuration.
localparam	split_data	=  14'h4;    // Split 32-bit Data value.
localparam	load_shft_data	=  14'h8;    // Load data from split_data state.
localparam	enable_sck	=  14'h10;   // Enable SCK.
localparam      wait_two        =  14'h20;   // Wait for two SCK cycles before CSn.
localparam      enable_cs	=  14'h40;   // Enable CSn by setting low.
localparam      wait_for_wr	=  14'h80;   // Wait for 18 Clock Cycles. 
localparam      disable_cs      =  14'h100;  // Disable CSn by setting high. 
localparam      set_load_en     =  14'h200;  // Set LOAD_EN high for one clock cycle.
localparam      wait_one        =  14'h400;  // Set LOAD_EN low and wait for one SCK cycle.
localparam      set_cfg_valid   =  14'h800;  // Toggle Configuration Valid high.
localparam      set_cfg_clk     =  14'h1000; // Toggle Configuration Clock high.
localparam      pause           =  14'h2000; // wait for 20 clk_50's in between CSn.

// Local Wire Declarations:
// N/A

// Local Register Declarations:
reg     [13:0]  cs;             // SPI Engine Current State Register.                                                                        
reg     [13:0]  ns;             // SPI Engine Next State Register.                                                                           
reg             clk_spi_pre2;   // clk_50 divide by 2 register. (~25 MHz)                                                          
reg             clk_spi_pre4;   // clk_50 divide by 2 register. (~12.5 MHz)                                                        
reg             clk_spi_pre8;   // clk_50 divide by 2 register. (~6.25 MHz)                                                        
reg             clk_spi_pre16;  // clk_50 divide by 2 register. (~3.125 MHz)    
reg             clk_spi;        // clk_50 divide by 2 register. (~1.5625 MHz)
reg             cfg_clk;
reg             cfg_vld;
reg             clk_spi_en;     // SPI Clock Enable. (Active High).     
reg             spi_load_en;    // SPI Load Enable for slave device. (Active High)

reg     [19:0]  spi_ld_data;    // SPI Load Data. Either Upper or Lower 18-bits.   // DEAN WAS HERE
reg     [20:0]  spi_shift_reg;  // Used to send control byte data to ADC.          // DEAN WAS HERE

reg             spi_shift_load; // SPI Data Load. (Active Low).                                                                   
reg             shift_done;     // Shift Done signal for shifting the SPI data out.
reg     [4:0]   shift_count;    // counts SCK cycles
reg             split_done;     // Split Done. Set high during 4th SPI transaction.
reg     [1:0]   split_cnt;      // Split Counter for counting 4 SPI transactions.
reg             n_cs_spi;

reg     [19:0]  upperd;  // DEAN WAS HERE
reg     [19:0]  lowerd;  // DEAN WAS HERE
reg     [19:0]  uppera;  // DEAN WAS HERE
reg     [19:0]  lowera;  // DEAN WAS HERE

reg             load_done;
reg     [2:0]   wait_2_cnt;
reg             wait_2_done;
reg     [1:0]   wait_1_cnt;
reg             wait_1_done;
reg             pause_done;
reg     [3:0]   pause_cnt;

// next state logic
always @ (cs or rst_n or start_spi or shift_done or split_done or hold or send or wait_1_done or wait_2_done or pause_done or load_done)
begin : NEXT_STATE_LOGIC
    if (!rst_n) 
	ns <= idle;
    else
        case (cs)
            idle:           ns <= wait_data;
            wait_data:      if (start_spi) ns <= split_data;
                            else if (send) ns <= idle;
			    else ns <= wait_data;
            split_data:     if (hold) ns <= load_shft_data;
			    else ns <= split_data;
            load_shft_data: if (split_done) ns <= wait_data;
                            else ns <= enable_sck;
            enable_sck:     ns <= wait_two;
	    wait_two:	    if (wait_2_done) ns <= enable_cs; // Wait for 8 100 MHz Clock Cycles. (3-bit counter)
			    else ns <= wait_two;
            enable_cs:	    ns <= wait_for_wr;
            wait_for_wr:    if (shift_done) ns <= disable_cs;
			    else ns <= wait_for_wr;
            disable_cs:     ns <= set_load_en;
            set_load_en:    if (load_done) ns <= wait_one;    // Wait for 4 100 MHz Clock Cycles. (2-bit counter)
                            else ns <= set_load_en;
	    wait_one:	    if (wait_1_done) ns <= set_cfg_valid; //pause; // Wait for 4 100 MHz Clock Cycles, then disable SCK. (2-bit counter)
		            else ns <= wait_one;

            set_cfg_valid:  ns <= set_cfg_clk;    
            set_cfg_clk:    ns <= pause;
            pause:          if ((pause_done == 1'b1) && (split_done == 1'b0)) ns <= load_shft_data; // Wait for 16 SCK cycles (64 100 MHz cycles).
                            else if ((pause_done == 1'b1) && (split_done == 1'b1)) ns <= wait_data;
                            else ns <= pause;            
            default:        ns <= idle;
    endcase
end

// next-state to current-state register
always @ (posedge clk_50 or negedge rst_n) 
begin : CS_TO_NS
    if (!rst_n) 
	cs <= idle;
    else 
	cs <= ns;
end

// state machine
always @ (posedge clk_50 or negedge rst_n)
begin : FSM
    if (!rst_n) 
    begin
        n_cs_spi <= 1'b1;
        clk_spi_en <= 1'b0;
        spi_load_en <= 1'b0;

        spi_ld_data <= 20'hF_FFFF;   // DEAN WAS HERE

        spi_shift_load <= 1'b0;
        hold <= 1'b1;

        upperd <= 20'h0_0000;   // DEAN WAS HERE
        lowerd <= 20'h0_0000;   // DEAN WAS HERE
        uppera <= 20'h0_0000;   // DEAN WAS HERE
        lowera <= 20'h0_0000;   // DEAN WAS HERE

        split_cnt <= 2'b00;
        split_done <= 1'b0;
        wait_1_done <= 1'b0;
        wait_1_cnt <= 2'h0;
        wait_2_done <= 1'b0;
        wait_2_cnt <= 3'h0;
        pause_done <= 1'b0;
        pause_cnt <= 4'h0;
        load_done <= 1'b0;
        cfg_clk <= 1'b0;
        cfg_vld <= 1'b0;
        send <= 1'b0;
    end
    else
        case (cs)
            idle:               begin
                                        n_cs_spi <= 1'b1;
                                        clk_spi_en <= 1'b0;
                                        spi_load_en <= 1'b0;

                                        spi_ld_data <= 20'hF_FFFF;  // DEAN WAS HERE

                                        spi_shift_load <= 1'b0;
                                        hold <= 1'b0;

                                        upperd <= 20'h0_0000;  // DEAN WAS HERE
                                        lowerd <= 20'h0_0000;  // DEAN WAS HERE
                                        uppera <= 20'h0_0000;  // DEAN WAS HERE
                                        lowera <= 20'h0_0000;  // DEAN WAS HERE

                                        split_cnt <= 2'b00;
                                        split_done <= 1'b0;
                                        wait_1_done <= 1'b0;
                                        wait_1_cnt <= 2'h0;
                                        wait_2_done <= 1'b0;
                                        wait_2_cnt <= 3'h0;
                                        pause_done <= 1'b0;
                                        pause_cnt <= 4'h0;
                                        load_done <= 1'b0;
                                        cfg_clk <= 1'b0;
                                        cfg_vld <= 1'b0;
                                        send <= 1'b0;
                                end
            wait_data:          begin
                                        //; // Do nothing but wait for data to start.
                                        hold <= 1'b0;
                                        
                                        if (split_done)
                                        begin
                                                send <= 1'b1;
                                                split_done <= 1'b0;
                                        end
                                        else
                                        begin
                                                send <= 1'b0;
                                        end
                                        
                                end
            split_data:         begin
                                        clk_spi_en <= 1'b0;
                                        spi_load_en <= 1'b0;
                                        n_cs_spi <= 1'b1;
                                        split_cnt <= 2'b00;
                                        
					upperd <= {1'b0,1'b1,config_data[35:18]};   // DEAN WAS HERE
					lowerd <= {1'b1,1'b1,config_data[17:0]};    // DEAN WAS HERE

					uppera <= {1'b0,1'b0,config_addr[35:18]};   // DEAN WAS HERE
					lowera <= {1'b1,1'b0,config_addr[17:0]};    // DEAN WAS HERE
                                                
                                        hold <= 1'b1;
                                end
            load_shft_data:     begin
                                        spi_load_en <= 1'b0;
                                        spi_shift_load <= 1'b1;
                                        clk_spi_en <= 1'b0;
                                        case (split_cnt)
                                                2'b00:   begin spi_ld_data <= uppera; split_done <= 1'b0; end
                                                2'b01:   begin spi_ld_data <= lowera; split_done <= 1'b0; end
                                                2'b10:   begin spi_ld_data <= upperd; split_done <= 1'b0; end
                                                2'b11:   begin spi_ld_data <= lowerd; split_done <= 1'b1; end
                                                default: begin spi_ld_data <= 20'h0_0000; split_done <= 1'b0; end      // DEAN WAS HERE
                                        endcase
                                        split_cnt <= split_cnt + 2'b01;
                                end
            enable_sck:         begin
                                        spi_shift_load <= 1'b1;
                                        clk_spi_en <= 1'b1;
                                end
            wait_two:           begin
                                        if (wait_2_cnt == 3'h4) // 3'h6
                                        begin
                                                wait_2_done <= 1'b1;
                                                wait_2_cnt <= 3'h0;
                                        end
                                        else
                                        begin
                                                wait_2_done <= 1'b0;
                                                wait_2_cnt <= wait_2_cnt + 3'h1;
                                        end
                                end
            enable_cs:          begin    
                                        spi_shift_load <= 1'b0;
                                        n_cs_spi <= 0;
                                end
            wait_for_wr:        begin // Wait for 18-bits of data to be written.
                                        n_cs_spi <= 0;
                                end
            disable_cs:         begin   
                                        n_cs_spi <= 1;
                                        spi_load_en <= 1'b1;
                                end
            set_load_en:        begin   
                                        spi_load_en <= 1'b1;
                                        load_done <= 1'b1;
                                end
            wait_one:           begin
                                        spi_load_en <= 1'b0;
                                        if (wait_1_cnt == 2'h3)
                                        begin
                                                wait_1_done <= 1'b1;
                                                wait_1_cnt <= 2'h0;
                                        end
                                        else
                                        begin
                                                wait_1_done <= 1'b0;
                                                wait_1_cnt <= wait_1_cnt + 2'h1;
                                        end
                                end
            set_cfg_valid:      begin
                                        clk_spi_en <= 1'b0;
                                        cfg_vld <= 1'b1;
                                        load_done <= 1'b0;
                                end
            set_cfg_clk:        begin
                                        cfg_clk <= 1'b1;                        
                                end
            pause:              begin
                                        clk_spi_en <= 1'b0;
                                        cfg_clk <= 1'b0;
                                        cfg_vld <= 1'b0;
                                        if (pause_cnt == 4'hF)
                                        begin
                                                pause_done <= 1'b1;
                                                pause_cnt <= 4'h0;
                                        end
                                        else
                                        begin
                                                pause_done <= 1'b0;
                                                pause_cnt <= pause_cnt + 4'h1;
                                        end
                                end
            default:            begin
                                        n_cs_spi <= 1'b1;
                                        clk_spi_en <= 1'b0;
                                        spi_load_en <= 1'b0;

                                        spi_ld_data <= 20'hF_FFFF;  // DEAN WAS HERE

                                        spi_shift_load <= 1'b0;
                                        hold <= 1'b1;

                                        upperd <= 20'h0_0000;  // DEAN WAS HERE
                                        lowerd <= 20'h0_0000;  // DEAN WAS HERE
                                        uppera <= 20'h0_0000;  // DEAN WAS HERE
                                        lowera <= 20'h0_0000;  // DEAN WAS HERE

                                        split_cnt <= 2'b00;
                                        split_done <= 1'b0; 
                                        wait_1_done <= 1'b0;
                                        wait_1_cnt <= 2'h0;
                                        wait_2_done <= 1'b0;
                                        wait_2_cnt <= 3'h0;
                                        pause_done <= 1'b0;
                                        pause_cnt <= 4'h0;
                                        load_done <= 1'b0;
                                        cfg_clk <= 1'b0;
                                        cfg_vld <= 1'b0;
                                        send <= 1'b0;
                                end
        endcase
end

// ****************** SPI Serial Interface **********************

// *** Generate SPI Clock ***
//  if clk_50_spi_en, toggle clk_50_spi:  Must be >= 500 ns (period)

// --- Divide-by-2 (25MHz) ---  
always @ (posedge clk_50 or negedge clk_spi_en)
    if (!clk_spi_en) clk_spi_pre2 <= 0;
    else clk_spi_pre2 <= ~clk_spi_pre2;

// --- Divide-by-4 (12.5MHz) ---    
always @ (posedge clk_spi_pre2 or negedge clk_spi_en)
    if (!clk_spi_en) clk_spi <= 0;
    else clk_spi <= ~clk_spi; 

    
/*    
// --- Divide-by-4 (25MHz) ---
always @ (posedge clk_spi_pre2 or negedge clk_spi_en)
    if (!clk_spi_en) clk_spi_pre4 <= 0;
    else clk_spi_pre4 <= ~clk_spi_pre4;

// --- Divide-by-8 (12.5MHz) ---        
always @ (posedge clk_spi_pre4 or negedge clk_spi_en)
    if (!clk_spi_en) clk_spi_pre8 <= 0;
    else clk_spi_pre8 <= ~clk_spi_pre8;

// --- Divide-by-16 (6.25MHz) ---      
always @ (posedge clk_spi_pre8 or negedge clk_spi_en)
    if (!clk_spi_en) clk_spi_pre16 <= 0;
    else clk_spi_pre16 <= ~clk_spi_pre16;       

// --- Divide-by-32 (f = 3.125MHz, t = 320ns) ---      
always @ (posedge clk_spi_pre16 or negedge clk_spi_en)
    if (!clk_spi_en) clk_spi <= 0;
    else clk_spi <= ~clk_spi; 

*/
// **************************

// *** Assign SCK, CSn and LOAD_EN ***
assign CSn = n_cs_spi;
assign LOAD_EN = spi_load_en;
assign SCK = clk_spi;

// *** load/shift spi data ***
//  1.  Load the data to be shifted into the shift register.
//  2.  Once the shift register is loaded, left shift the 
//      data out one bit at a time.

//always @ (posedge clk_spi or negedge rst_n) // or posedge spi_shift_load)
always @ (negedge clk_spi or negedge rst_n)        
begin : SPI_SR
    if (!rst_n) 
            spi_shift_reg <= 21'h1F_FFFF;     // DEAN WAS HERE
    else if (spi_shift_load) 
            // Data gets latched onto spi_shift_reg when spi_shift_load is low.
            spi_shift_reg <= {1'b1,spi_ld_data}; 
    else if (!CSn)
            spi_shift_reg <= {spi_shift_reg[19:0], 1'b0};       // DEAN WAS HERE
//            spi_shift_reg <= spi_shift_reg << 1;     
end

assign MOSI = (CSn | shift_done) ? 1'b1 : spi_shift_reg[20];

// *** Assign Configuration Clock and Valid ***
assign config_clk = cfg_clk;
assign config_valid = cfg_vld;

// ***************************************************************


// inc shift_count: As data is shifted out to the ADC, count the clock edges.
//                  This will tell us how many bits have been shifted out.
always @ (negedge SCK or negedge clk_spi_en)
begin : SHIFT_COUNTER
    if (!clk_spi_en) 
        begin
            shift_count <= 0;
            shift_done <= 0;
        end
    else 
        begin
                if (shift_count == 5'd22)    // DEAN WAS HERE //pm
                begin
                        shift_done <= 1'b1;
                        shift_count <= 5'h00;
                end
                else
                begin
                        shift_done <= 1'b0;
                        shift_count <= shift_count + 5'h01;
                end
        end
end

always @ (negedge SCK or negedge clk_spi_en)
begin : MISO_IN
        if (!clk_spi_en)
        begin
                miso_read[19:0] <= 20'h0_0000;  // DEAN WAS HERE
        end
        else
        begin
                if (!config_valid)
                        miso_read[19:0] <= {miso_read[18:0],MISO};  // DEAN WAS HERE
                else
                        miso_read[19:0] <= miso_read[19:0];  // DEAN WAS HERE
        end
end

assign miso_rdy = (CSn) ? 1'b1 : 1'b0;

endmodule

/*

// *** Instantiate the SPI Master Engine ***
spie    spie   (.clk_50 (clk_50),
	        .rst_n (rst_n),
          	.MISO (MISO),
	        .config_data (config_data),
                .config_addr (config_addr),
                .start_spi (start_spi),
                .hold (hold),
                .send (send),

                .miso_rdy (miso_rdy),
                .config_clk (config_clk),
                .config_valid (config_valid),
                .SCK (SCK),
                .CSn (CSn),
                .MOSI (MOSI),
                .LOAD_EN (LOAD_EN),
                .miso_read (miso_read));

*/
